#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Victron2PVO
# requests to stenis@t-online.de
# This script collects data from Venus GX and send them every 5 minutes to PV Output https://pvoutput.org/
# you need at least python2.7, install requests and pyModbusTCP via PIP
# you should be familar in configuration of PV Output, specially extended values
# a donated PVO account is maybe required
# script is running on Synology NAS, should also run on Raspi
# run script on Synology in task planner, user defined as start up 
# acitvate modbus in Venus GX 
# on runtime log is written in logpvo.txt
# Energy values are stored in pvoval.txt in case of restart

import time
import sys
import requests
from datetime import datetime
from pyModbusTCP.client import ModbusClient

PVOUTPUT = "https://pvoutput.org/service/r2/addstatus.jsp"
APIKEY = "xxxx" # get API Key from settings
SYSTEMID = "xxxx" # add new system and take this id
LOKALPATH = "/volume1/homes/xxxxx/" # path of stored script
VENUS_IP = "192.168.178.xxxx" # Ip of the Venus GX

# modify all values with xxxx

c = ModbusClient(host=VENUS_IP, port=502, unit_id=100, auto_open=True) # power values
d = ModbusClient(host=VENUS_IP, port=502, unit_id=225, auto_open=True) # temperature from battery (here: pylontech) 

sys.stdout = open(LOKALPATH + 'logpvo.txt', 'w') # logging

dateTimeObj = datetime.now()
ts = dateTimeObj.strftime("%d-%b-%Y %H:%M:%S")
print(ts, 'Start')

heute =  time.strftime("%Y%m%d")

def upload(v1,v2,v3,v4,v5,v7,v8,v9,v10):

    now = datetime.now()
    payload = {
        "d": now.strftime("%Y%m%d"),
        "t": now.strftime("%H:%M"),
        "v1": int(v1),                # PV Energy
        "v2": int(v2),                # PV Power
        "v3": int(v3),                # Consumption Energy   
        "v4": int(v4),                # Consumption Power
        "v5": v5,                     # Temperature of battery
        "v7": int(v7),                # Grid (extended data in PVO)
        "v8": int(v8),                # Battery discharge (extended data in PVO)
        "v9": int(v9),                # Battery charge (extended data in PVO)
        "v10": int(v10),              # Battery SOC (extended data in PVO)
        "c1": 1
    }
    try:
        requests.post(PVOUTPUT,
            headers={
                "X-Pvoutput-Apikey": APIKEY,
                "X-Pvoutput-SystemId": SYSTEMID
            }, data=payload)
    except:
        pass

    return True

f = open(LOKALPATH + "pvoval.txt", "r")
x =f.read().split(':')
v1= int(x[0])
v3= int(x[1])
# reads the last generation values in case of script restart 

start = datetime.now()
Anfang = datetime.now()
netz_ws = 0
pv_ws=0
verbrauch_ws=0
battery_wsp=0
battery_wsn=0
v2=0
v4=0
v5=0
v7=0
v8=0
v9=0
v10=0
SOC=0

while True:

    try:
        bits = c.read_input_registers(811,12)
        
        if bits != None:

            N1 = bits[9]-65536 if bits[9] > 32668 else bits[9]	
            N2 = bits[10]-65536 if bits[10] > 32668 else bits[10]	
            N3 = bits[11]-65536 if bits[11] > 32668 else bits[11]

            P1 = bits[0]-65536 if bits[0] > 32668 else bits[0]	
            P2 = bits[1]-65536 if bits[1] > 32668 else bits[1]	
            P3 = bits[2]-65536 if bits[2] > 32668 else bits[2]
    
            C1 = bits[6]-65536 if bits[6] > 32668 else bits[6]	
            C2 = bits[7]-65536 if bits[7] > 32668 else bits[7]	
            C3 = bits[8]-65536 if bits[8] > 32668 else bits[8]

        if bits != None:
        
            bits = c.read_input_registers(866,1)
            B = bits[0]-65536 if bits[0] > 32668 else bits[0] # chargepower; take this for charge

        if bits != None:

            bits = c.read_input_registers(842,2)
            SOC = bits[1]
            BP = bits[0]-65536 if bits[0] > 32668 else bits[0] # Battery power, take this for discharge

            PV = P1 + P2 + P3               # total PV
            Netz = (N1 + N2 + N3) * -1      # total grid (reverse negative values)
            Verbrauch = C1 + C2 + C3        # total consumption
    
            ende = datetime.now()
    
            intervall = ende - start
            Laufzeit = ende - Anfang
    
            netz_ws = netz_ws + intervall.total_seconds() * Netz
            pv_ws = pv_ws + intervall.total_seconds() * PV
            verbrauch_ws= verbrauch_ws + intervall.total_seconds() * Verbrauch
    
            if B >= 0:
                battery_wsp = battery_wsp + intervall.total_seconds() * B # charge
            if B < 0:
                battery_wsn = battery_wsn + abs(intervall.total_seconds() * BP) # discharge
    
    except OSError as err:
        print("OS error: {0}".format(err))
    
    start = datetime.now()
    
    if Laufzeit.total_seconds() >= 299:
        bits = d.read_input_registers(262,1)
        rt= Laufzeit.total_seconds()

        v1 = v1 + (pv_ws / 3600)
        v2 = pv_ws / rt
        v3 = v3 + (verbrauch_ws / 3600) + (battery_wsp / 3600)
        v4 = (verbrauch_ws / rt) + (battery_wsp / rt)
        
        if bits != None:
            v5 = float((bits[0]-65536) if bits[0] > 32668 else bits[0])/10
        
        v7 = netz_ws / rt
        v8 = battery_wsn / rt #discharge
        v9 = abs(battery_wsp / rt) #charge
        v10 = SOC
        
        upload(v1,v2,v3,v4,v5,v7,v8,v9,v10)
        
        dateTimeObj = datetime.now()
        ts = dateTimeObj.strftime("%d-%b-%Y %H:%M:%S")

        print(ts, int(v1), int(v2), int(v3), int(v4), v5, int(v7), int(v8), int(v9), int(v10))
        netz_ws=0
        pv_ws=0
        verbrauch_ws=0
        battery_wsp=0
        battery_wsn=0
        # save generation values to disk 
        f = open(LOKALPATH + "pvoval.txt", "w")
        f.write(str(int(v1)) + ':' + str(int(v3))) 
        sys.stdout.flush()
        Anfang = datetime.now() # next cycle   
   
    aktuell = time.strftime("%Y%m%d")   # change of day
    if heute != aktuell: 
        heute = time.strftime("%Y%m%d")
        v1=0    # reset of energy on next day
        v3=0    # reset of energy on next day
        
    if Laufzeit.total_seconds() > 290: # running last 10 sec. faster to hit the 5 minute interval more exactly
        tsl=1
    else:
        tsl=5 # get modbus values every 5 seconds
            
    time.sleep(tsl)



